;-------------------------------------------------------------------------
; Copyright (C) 2009  Tuan Do, Mike Fitzgerald, Luke Johnson
;
;     This program is free software: you can redistribute it and/or modify
;     it under the terms of the GNU General Public License as published by
;     the Free Software Foundation, either version 3 of the License, or
;     (at your option) any later version.
;
;     This program is distributed in the hope that it will be useful,
;     but WITHOUT ANY WARRANTY; without even the implied warranty of
;     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;     GNU General Public License for more details.
;
;     You should have received a copy of the GNU General Public License
;     along with this program.  If not, see
;     <http://www.gnu.org/licenses/>.
;
;     NOTE: only this file (fgui.pro) is released under GPL, contact
;     the corresponding authors of the other procedures for copyright
;     information. 
;-------------------------------------------------------------------------

;+
; PURPOSE: GUI for the Fourier Optics lab. For educational purposes
; only. Not tested for research applications.
;
; $Id: fgui.pro 3764 2007-07-30 22:34:22Z fitz $
;                    2007-07-30 - T. Do - sliders now update Zernike
;                                 coeff. array.
;                    2007-08-07 - T. Do CfAO summer school code for
;                                 vision and fourier optics lab
;                    2007-08-13 - T. Do - initial public code release
;                    2008-08-15 - T. Do - minor update
;                    2009-08-18 - v1.1 
;                               - L. Johnson:
;                               - updated disp.pro 
;                               - Windows now update the display range
;                                 automatically, no need to close and reopen
;                               - The calculate button now scales the
;                                 zernike terms to the selected pupil
;                                 size instead of always using a 5mm
;                                 pupil. 
;                               - T. Do:
;                               - added menu to display the README file.
;                               
;                    2010-08-16 - L. Johnson:
;                               - Added strehl display to psf window and rms display to wavefront window
;                               - Changed units so sliders are in nanometers and wavefront display in microns
;                               - Added wavelength setting
;                               - Fixed Kolmogorov phase screen to make it more accurate
;                               - Increased pupil size to 10 mm max
;                               - Deleted resize in convolve function to speed it up: new pixel scale is approx 7 pixels per arcminute
;                               - Nicole resized the sample E images and the eye chart images to reflect this change in plate scale
;                               
;-


; -------------- helper functions ---------------
common fvars, aperture_ind, psf, phase, zernCoef, Dr0, grid, arrs

; returns true if variable is odd
function isodd, a
  return, a mod 2
end

; get polar coordinates from x,y arrays
pro get_polar, x, y, r, th
  r = sqrt(x^2+y^2)
  th = atan(y, x)
end

; make_grid, n
;
; create a grid from -max to +max with n points on a side
; 
; if ny given, creates a rectangular grid with -max to +max in the
; longest dimension
;
; center will contain the pixel coordinates of the grid origin.
; default is the center of the pixels
function make_grid, n, ny, max = max, center = center
  
  m = dblarr(2)+1.d
  if (n_params() gt 1) then begin
    m[1] = m[1] * double(ny)/double(n)
    m = m / max(m)
    if not keyword_set(center) then center = double([n-1, ny-1])/2.
  endif else if not keyword_set(center) then center = double(n-1)/2.
  if keyword_set(max) then begin
    if (n_elements(max) gt 1) then m = max $
    else m = m * max
  endif

;  x = dindgen(n)/(double(n-1)/2.) - 1.
  x = (dindgen(n)-center[0])/(double(n-1)/2.)
  ox = dblarr(n)+1.

  if (n_params() gt 1) then begin
;    y = dindgen(ny)/(double(ny-1)/2.) - 1.
    y = (dindgen(ny)-center[1])/(double(ny-1)/2.)
    oy = dblarr(n)+1.
  endif else begin
    y = x
    oy = ox
    ny = n
  endelse

  x = x*m[0]
  y = y*m[1]

  xx = x#ox
  yy = oy#y

  get_polar, x, y, r, th
  get_polar, xx, yy, rr, thth

  return, {n:n*n, nx:n, ny:ny, x:x, y:y, $
           xx:xx, yy:yy, r:r, rr:rr, th:th, thth:thth, $
           scale:2.*m/double([n, ny])}
end

; swap the quadrants around in an array; useful for keeping PSFs, etc.
; centered during FFT operations
function swap_quadrants, u
  sz = size(u)  &  nx = sz[1]  &  ny = sz[2]
  n = u*0d                      ; preserve float/complex
  n[0, 0] = u[nx/2:nx-1, ny/2:ny-1]
  n[nx/2, 0] = u[0:nx/2-1, ny/2:ny-1]
  n[0, ny/2] = u[nx/2:nx-1, 0:ny/2-1]
  n[nx/2, ny/2] = u[0:nx/2-1, 0:ny/2-1]
  return, n
end

; compute a PSF given a phase map (radians) and an amplitude map
; (e.g. pupil function).  zp_fact is the zero-padding factor (default
; 2)
function phase_to_PSF, phase, amplitude, zp_fact = zp_fact

  if ~keyword_set(zp_fact) then zp_fact = 2
  sz = size(phase)  &  nx = sz[1]  &  ny = sz[2]

  ;; compute complex amplitude
  complex_amp = amplitude * exp(dcomplex(0., phase))

  ;; place complex amplitude into zero-padded array
  zca = dcomplexarr(zp_fact*nx, zp_fact*ny)
  zca[(zp_fact-1)*nx/2, (zp_fact-1)*ny/2] = complex_amp
  
  ;; compute PSF
  return, swap_quadrants(abs(fft(zca, /double))^2)
end

; compute an OTF from a PSF
function PSF_to_OTF, psf
  return, swap_quadrants(fft(swap_quadrants(psf), /inverse, /double))
end


; zernikes
;

; returns n and m for a given j
pro zernike_getnm, j, n, m
  n = fix(.5*(sqrt(8.*j-7.)-1.))
  m = ((j - n*(n+1)/2 - (n mod 2)) / 2) * 2 + (n mod 2)
  return
end

; get_r
;
; returns the sum of powers of r for the zernike polynomial evaluation
function get_r, n, m, r
  if isodd(n-m) then return, 0.
  acc = 0.
  for s = 0, (n-m)/2 do $
    acc = acc + $
          (-1)^s*factorial(n-s)/(factorial(s)*factorial((n+m)/2-s)*factorial((n-m)/2-s)) * $
          r^(n-2*s)
  return, acc
end

; eval_zernike
;
; evaluate a zernike polynomial (specified by j,n,m) at polar
; coordinates r, theta
function eval_zernike, j, n, m, r, th, norm = norm

  c = get_r(n, m, r)

  if (m ne 0) then begin
    c = c*sqrt(2.)
    if (j mod 2 eq 0) then c = c*cos(m*th) $
    else c = c*sin(m*th)
  endif

  z = sqrt(n+1.)*c

  if keyword_set(norm) then return, z*1. ; FIXME normalization factor
  return, z
end

; get_zernike
;
; evaluate a zernike polynomial (index j) at polar coordinates r,
; theta
function get_zernike, j, r, th, norm = norm
  zernike_getnm, j, n, m
  return, eval_zernike(j, n, m, r, th, norm = norm)
end

; evaluate zernike coefficients at given spatial
; coordinates -- ignore piston
function eval_zernikes, n, r, th
  s = size(r)
  ni = s[1] & nj = s[2]
  a = dblarr(ni, nj, n)
  for j = 0, n-1 do begin
    z = get_zernike(j+2, r, th)
    a[0, 0, j] = z
  endfor

  return, a
end


; kolmogorov  (code from original fourier optics computer lab)
;

function ft,a,inverse=inverse
  n = (size(a))(1)
  if keyword_set(inverse) then $
    r = shift(fft(shift(a,n/2,n/2),/inverse),n/2,n/2) $
  else $
    r = shift(fft(shift(a,n/2,n/2)),n/2,n/2)
  return,r
end

;+
;  phasescreen.pro - simulate Kolmogorov turbulence
;
;  creates a Gaussian random phase screen with the structure function
;    D(r) = 6.88 (r/r0)^(5/3)
;
;  usage:
;     screen = phasescreen(n,m,dx,r0,filter,seed,NEW=new)
;   where
;       n and m - dimensions of the array
;       dx - the physical size of a pixel (meters)
;       r0 - Fried's parameter (meters)
;       filter - the spatial filter function for the white noise
;           This is generated if the keyword NEW is set
;           Otherwise it assumed to be pre-generated
;           It must be regernerated if any of n,m,dx, or r0 is changed
;       seed - the random number seed
;       new - keyword to trigger a new filter calculation
;-
function phasescreen,n,m,dx,r0,filter,seed,new=new
   dkx = 1./(n*dx)
   dky = 1./(m*dx)
   if keyword_set(new) then begin
     kx = (findgen(n)-n/2) # (1+fltarr(m)) * dkx
     ky = (1+fltarr(n)) # (findgen(m)-m/2) * dky
     ksq = kx^2 + ky^2
     ksq(n/2,m/2) = 1.
     ;filter = sqrt(.023 * r0^(-5./3.) * ksq^(-11./6.) ) ;this is the old filter, it gives weird results
     filter = sqrt(0.023)*(2*!pi)^(5./6.)*r0^(-5./6.)*ksq^(-11./12.)*sqrt(dkx*dky)
     filter(n/2,m/2) = 0.
   endif
   w = randomn(seed,n,m) ; white noise
   wf = sqrt(float(n)*float(m))*ft(w)/sqrt(dkx*dky)
   sf = wf*filter
;   s = real(ft(sf,/inverse)*dkx*dky)
   s = abs(ft(sf,/inverse)*dkx*dky)
   ; put in units of centiradians
   return,s
end


; pupils
;

; circular pupil with radius = 0.5 m
function circ_pupil, xx, yy, r, th
  return, r le 0.5
end

; square pupil with side length = 1/2/sqrt(2) m
function square_pupil, xx, yy, r, th
  return, (abs(xx) le 1d/2d/sqrt(2)) and (abs(yy) le 1d/2d/sqrt(2))
end

; hexagonal pupil with longest dimension = 0.5 m
function hex_pupil, xx, yy, r, th
  return, r le 0.5*cos(30.*!dtor)/cos((th+!dpi) mod (60*!dtor) - 30*!dtor)
end

function tri_pupil, xx, yy, r, th
;;  return, r le 0.5*cos(60.*!dtor)/cos((th+!dpi) mod (120*!dtor) -  30*!dtor)
  return, ((sqrt(3)*abs(xx)+yy) le .3) and (yy ge -.3)
  
end

function slit_pupil, xx, yy, r, th
  dx = 0.1
  return, (abs(xx) le dx) and (abs(yy) le sqrt(0.5^2-dx^2))
end


; ---------------- GUI code ---------------------

pro fgui_event, ev
;; event handler for the Fourier optics GUI
  common fvars

  widget_control, ev.top, get_uvalue = infoptr

  info = *infoptr
  
  
  widget_control, info.lambda0, get_value = wavelength_nm
  if info.wavelength_nm ne wavelength_nm then begin
    ;if wavelength has changed then we need to re-convert nanometers to centiradians
    zernCoef *= info.wavelength_nm/wavelength_nm
    info.wavelength_nm = wavelength_nm
    ;also need to change scaled pupil
    redo_ap=1
  endif else redo_ap = 0
  
  
  if ev.id eq info.file_exit then begin
    widget_control, ev.top, /destroy

    ;; check to see if display widgets are running and destroy
;     if info.psfDisp then begin
;        dispTop = widget_info([info.psfDisp, info.phaseDisp, info.mtfDisp], /parent)
;        widget_control, dispTop, /destroy
;     endif
  endif

  if (ev.id eq info.apselect) or (ev.id eq info.spupil) or (redo_ap) then begin
     widget_control, info.apselect, get_value = aperture_ind
;     aperture_ind = ev.value
     widget_control, info.spupil, get_value = pupilSize
     scale = pupilSize/10.0*830./info.wavelength_nm   ;; scale aperture by pupil size and wavelength

     arrs.aperture = call_function(info.aperture_funcs[aperture_ind, 1], $
                                   grid.xx/scale, grid.yy/scale, grid.rr/scale, grid.thth)
     ;; display the aperture
     widget_control, info.apdraw, get_value = apwin
     wset, apwin
     imdisp, arrs.aperture
     info.diffmax = max(phase_to_PSF(fltarr(size(arrs.aperture,/dimensions)), arrs.aperture)) ;maximum of diffraction limited psf
  endif

  if ev.id eq info.pslide then $
     Dr0 = ev.value

  if ev.id eq info.pCalc then begin
    ;; put in function for phase error calculations here
    if (Dr0 gt 0.) then $
       arrs.phase = phasescreen(grid.nx, grid.ny, 1./grid.nx, 1./Dr0, filter, seed, /new) $ ; FIXME  save filter?
    else arrs.phase = dblarr(grid.nx, grid.ny)
  endif

  ;; updating coefficients, coefficients are in units of centiradians but slider inputs are now in nanometers
  ;doing wavelength conversion here so we should recalculate all coefficients in case wavelength has changed
  if ev.id eq info.sfocus then zernCoef[0] = ev.value*200.*!pi/info.wavelength_nm
  if ev.id eq info.sAst22m1 then zernCoef[1] = ev.value*200.*!pi/info.wavelength_nm
  if ev.id eq info.sAst221 then zernCoef[2] = ev.value*200.*!pi/info.wavelength_nm
  if ev.id eq info.sCom31m1 then zernCoef[3] = ev.value*200.*!pi/info.wavelength_nm
  if ev.id eq info.sCom311 then zernCoef[4] = ev.value*200.*!pi/info.wavelength_nm
  if ev.id eq info.sCom33m1 then zernCoef[5] = ev.value*200.*!pi/info.wavelength_nm
  if ev.id eq info.sCom331 then zernCoef[6] = ev.value*200.*!pi/info.wavelength_nm
  if ev.id eq info.sSpher40 then zernCoef[7] = ev.value*200.*!pi/info.wavelength_nm
  if ev.id eq info.sSpher421 then zernCoef[8] = ev.value*200.*!pi/info.wavelength_nm
  if ev.id eq info.sSpher42m1 then zernCoef[9] = ev.value*200.*!pi/info.wavelength_nm
  if ev.id eq info.sSpher44m1 then begin
     zernCoef[10] = ev.value*200.*!pi/info.wavelength_nm
     print, zernCoef
  endif
  if ev.id eq info.sSpher441 then zernCoef[11] = ev.value*200.*!pi/info.wavelength_nm
  
  if (ev.id eq info.zCalc) or (ev.id eq info.lambda0) then begin
    ;; calculate phase error based upon Zernike coefficients
    ;; the variable 'zernCoef' has all the coefficients in order
    n = n_elements(zernCoef)
    zc = dblarr(n+2)
    zc[2] = zernCoef
    zc = zc/100.0
    widget_control, info.spupil, get_value = pupilSize
    scale = pupilsize/10.0*830./info.wavelength_nm
    zerns = eval_zernikes(n+2, 2*grid.rr/scale, grid.thth)

;     circap = circ_pupil(grid.xx, grid.yy, grid.rr, grid.thth)
;     w = where(circap, nw)
;     for i = 0, n+1 do begin
;        x = zerns[*, *, i]
;        zerns[*, *, i] /= sqrt(total(x[w]^2)/double(nw))
;     endfor
    arrs.phase *= 0d
    for i = 0, n+1 do arrs.phase += zerns[*, *, i]*zc[i]
  endif

  ;; open file dialog
  if ev.id eq info.open_file then begin
     zernFile = dialog_pickfile(filter = '*.txt', $
                                title = 'Open Zernike File', $
                                path = info.zpath, get_path = newpath)
     info.zpath = newpath
;                                dir = '/Users/tdo/Documents/cfao/2007LAO_inquiry/CfAO_SummerSchool_2007/AbReport')
     if file_test(zernFile) then begin
        zernCoef = readzern2(zernFile);read in zernikes in microns
        zernCoef = fix(zernCoef*1000);convert to nm        
        for i = 0, n_elements(zernCoef)-1 do begin
           ;; change all sliders
           widget_control, info.sliders[i], set_value = zernCoef[i]
        endfor
        zernCoef = (zernCoef*2*!dpi)/info.wavelength_nm*100. ;; convert to centiradians
     endif
  endif
  
  ;; reset all the bars
  if ev.id eq info.zreset then begin
     for i = 0, n_elements(zernCoef)-1 do widget_control, info.sliders[i], set_value = 0
     zernCoef = zernCoef *0.0
  endif

  ;; convolve with image
  if ev.id eq info.file_conv then begin
     
     imgfile = dialog_pickfile(filter = ['*.tif','*.tiff'], path = info.impath, get_path = newpath)
     info.impath = newpath

     if file_test(imgfile) then begin
        ;; load tiff file
        im = read_tiff(imgfile)
        s = size(im)
        if s[0] gt 2 then im = reform(im[0, *, *])
        s = size(im)
        im = reverse(rot(im*1.0, 180))
        ;; we want to rebin to some standard size
        ;im = rebin(im, ceil(2560/s[1])*s[1], ceil(2560/s[1])*s[2])
        ;
        ;I don't understand the rebinning, it's going to slow down the convolution if the image is smaller than 2560. It's also going to change the pixel scale since you're resizing the image but not the psf.
        ;Also, the choice of 2560 is weird, it should be a power of 2 for most efficient computation
        ;I'm going to rebin only if the image is bigger than 2048 in one dimension and use congrid for better accuracy. LCJ 07-28-2010
        if s[1] gt 2048 or s[2] gt 2048 then begin
          scale = max([s[1]/2048.,s[2]/2048.])
          im = congrid(im, s[1]/scale, s[2]/scale,cubic=-0.5)
        endif
                 
        ;; convolution may take a long time, so make a status screen
        ;; pop up.
        convBase = widget_base(title = 'Convolving ...')
        convStatus = widget_label(convBase, value = 'Please wait, convolving: '+imgfile)
        widget_control, convBase, /realize
        
        ;; do the convolution
        im = convolve(im, arrs.PSF)
        ;; close the convolution status window
        widget_control, convBase, /destroy

        ;; display the results
        disp, im
        
     endif
  endif


  if (ev.id eq info.apselect) or (ev.id eq info.pCalc) or (ev.id eq info.zCalc) $
     or (ev.id eq info.spupil) or (ev.id eq info.lambda0) then begin
    ;; re-compute data
    ;need to scale pupil size with wavelength to get correct plate scale on PSF
    ;originally this was designed for 830nm so we're going to normalize to that
    
    arrs.PSF = phase_to_PSF(arrs.phase, arrs.aperture)
    arrs.OTF = PSF_to_OTF(arrs.PSF)
   
    info.strehl_ratio = max(arrs.PSF)/info.diffmax
    
    ;; get IDs
    phaseID = info.phaseDisp
    psfID = info.psfDisp
    mtfID = info.mtfDisp

    ;; re-draw
    phaseIm = arrs.aperture*arrs.phase
    phaseIm -= total(phaseIm)/total(arrs.aperture)
    phaseIm *= arrs.aperture
    
    info.rms_phase_rad = rms(phaseIm,arrs.aperture)
    rms_phase_um = info.rms_phase_rad*info.wavelength_nm/2000./!pi
    if total(phaseIm) eq 0 then phaseIm[0, 0] = 1
    fgui_draw, phaseID, phaseIm*info.wavelength_nm/2000./!pi, 'phase map'
    dispWidget_printtext,phaseID,0,0,'rms = '+string(rms_phase_um,format='(d0.3)')+' microns'
    
    fgui_draw, psfID, sqrt(arrs.PSF > 0), 'PSF'
    dispWidget_printtext,psfID,0,0,'Strehl = '+ string(info.strehl_ratio,format='(d0.3)')
    
    fgui_draw, mtfID, abs(arrs.OTF),'MTF'

    ;; put in widget IDs
    info.phaseDisp = phaseId
    info.psfDisp = psfId
    info.mtfDisp = mtfID

 endif

  ;; display the text of the README file
  if ev.id eq info.readme then begin
     xdisplayfile, 'README'
  endif 

  ;; save state
  *infoptr = info

end

pro fgui_draw, id, data, name
  id = long(id)
  if (widget_info(id, /valid_id) eq 0) then begin
     disp, data, name, id = newid
     id = newid
  endif else begin
     disp, data, name, id = id
     dispWidget_redrawDisplay, id, /forcesecondarydraw
  endelse
end

pro fgui

  common fvars

  if xregistered('fgui') then begin
    print, 'fgui is already running'
  endif else begin

    ;; trying to correct for 16 or 24 bit colors
    device, get_visual_depth = depth
    
    ;; to display on 24-bit colors correctly
    if (depth eq 24) or (depth eq 16) then  Device, Decomposed = 0 

    ;; aperture function labels and names
    aperture_funcs = [['Circle', 'Square', 'Hexagon', 'Slit', 'Triangle'], $
                      ['circ_pupil', 'square_pupil', 'hex_pupil', 'slit_pupil', 'tri_pupil']]


    base = widget_base(title = 'Fourier Optics GUI v1.1', /column, mbar = bar, $
                       xsize = 500)
    
    menu_base = widget_button(bar, value = 'File', /menu)
    open_file = widget_button(menu_base, value = 'Open Zernikes File ...', uname = 'open_file')    
    file_conv = widget_button(menu_base, value = 'Convolve File ... ')
    file_exit = widget_button(menu_base, value = 'Exit')

    help_base = widget_button(bar, value = 'Help', /menu)
    readme = widget_button(help_base, value = 'README', uname = 'readme')

    ;; make aperture column
    aptab = widget_tab(base)
    apbase_top = widget_base(aptab, /row, title = 'Aperture')
    apbase = widget_base(apbase_top, /column)
;    apselect = widget_combobox(apbase, value = ['Circular', 'Square',
;    'Hexagonal'])
    lambda0 = cw_field(apbase, title = 'Wavelength (nm)',/column, /floating, /return_events, value = 830.0)
    apselect = cw_bgroup(apbase, aperture_funcs[*, 0], /column, label_top = 'Select aperture', /exclusive, /frame, set_value = 0, xsize = 140)
    
    
    
    ;; make the draw widget
    apdraw = widget_draw(apbase_top, xsize = 128, ysize = 128)
    spupil = cw_fslider(apbase, /edit, title = 'Pupil Size (mm)', min = 0.1, max = 10.0, value = 5.0)

    ;; make phase tab
    pbase = widget_base(aptab, title = 'Phase Error ', /column, /base_align_center)
    plabel = widget_label(pbase, value = 'Kolmogorov turbulence phase errors')
    pslide = cw_fslider(pbase, /edit, title = 'D/r0', min = 0, max = 10, xsize = 200)
    pCalc = widget_button(pbase, value = 'Calculate')

    ;; make zernike tab
    zbase = widget_base(aptab, /row, title = 'Zernikes')
    zbase1 = widget_base(zbase, /column, xsize = 240)
    zbase2 = widget_base(zbase, /column, xsize = 240)
;    zlabel = widget_label(zbase1, value = "Zernike Coefficients")

    smax = 2000
    smin = -2000
    
    
    sfocus = widget_slider(zbase1, title = 'Focus (2,0)', min = smin*3, max = smax*3, value = 0)
    sAst22m1 = widget_slider(zbase1, title = 'Astigmatism, 45 deg (2,-2)', min = smin, $
                             max = smax, value = 0)
    sAst221 = widget_slider(zbase1, title = 'Astigmatism, 90 deg (2,2)', min = smin, max = smax, $
                            value = 0)
    sCom31m1 = widget_slider(zbase1, title = 'Vertical Coma (3,-1)', min = smin, $
                             max = smax, value = 0)
    sCom311 = widget_slider(zbase1, title = 'Horizontal Coma (3,1)', min = smin, max = smax, $
                            value = 0)
    sCom33m1 = widget_slider(zbase1, title = 'Trefoil (3,-3)', min = smin, $
                            max = smax, value = 0)
    sCom331 = widget_slider(zbase1, title = 'Trefoil (3,3)', min = smin, $
                            max = smax, value = 0)
    zCalc = widget_button(zbase1, value = 'Calculate')


    sSpher40 = widget_slider(zbase2, title = 'Spherical (4,0)', min = smin, $
                             max = smax, value = 0)
    sSpher421 = widget_slider(zbase2, title = '2nd Order Astigmatism, 90 deg (4,2)', min = smin, $
                              max = smax, value = 0)
    sSpher42m1 = widget_slider(zbase2, title = '2nd Order Astigmatism, 45 deg (4,-2)', min = smin, $
                               max = smax, value = 0)
    sSpher44m1 = widget_slider(zbase2, title = 'Quadrafoil (4,-4)', min = smin, $
                               max = smax, value = 0)
    sSpher441 = widget_slider(zbase2, title = 'Quadrafoil (4,4)', min = smin, $
                              max = smax, value = 0)

    zReset = widget_button(zbase2, value = 'Reset All')

    sliders = [sfocus, sAst22m1, sAst221, sCom31m1, sCom311, sCom33m1, sCom331, sSpher40, sSpher421, $
               sSpher42m1, sSpher44m1, sSpher441]
    ;; realize/make the gui
    widget_control, base, /realize
    xmanager, 'fgui', base, /no_block

    ;; initialize some variables
    aperture_ind = 0            ; 0 - cir 1 - square 2 - hex
    zernCoef = intarr(12)       ; zernike polynomials
    Dr0 = 0d                    ; D/r0

    n = 256                     ; number of points on a side

    grid = make_grid(n, max = 0.5) ; 1 m on a side
    aperture = circ_pupil(grid.xx, grid.yy, (grid.rr)*2, grid.thth)
    phase = dblarr(n, n)
    PSF = dblarr(2*n, 2*n)
    OTF = dcomplexarr(2*n, 2*n)
    arrs = {aperture:aperture, phase:phase, PSF:PSF, OTF:OTF}
    
    strehl_ratio = float(0)
    diffmax = double(max(phase_to_PSF(fltarr(size(aperture,/dimensions)), aperture))) ;maximum of diffraction limited psf)
    rms_phase_rad = float(0)
    ;; draw the initial aperture
     widget_control, apdraw, get_value = apwin
     wset, apwin
     imdisp, arrs.aperture
     
    widget_control, lambda0, get_value = wavelength_nm
    
    ;; info block to pass around
    info = {base:base, $
            file_exit: file_exit, $
            apselect:apselect, $
            pslide:pslide, $
            pcalc:pCalc, $
            sfocus:sfocus, $
            sAst221:sAst221, $
            sAst22m1:sAst22m1, $
            sCom311:sCom311, $
            sCom31m1:sCom31m1, $
            sCom331:sCom331, $
            sCom33m1:sCom33m1, $
            sSpher40:sSpher40, $
            sSpher421:sSpher421, $
            sSpher42m1:sSpher42m1, $
            sSpher441:sSpher441, $
            sSpher44m1:sSpher44m1, $
            zCalc:zCalc, $
            aperture_funcs:aperture_funcs, $
            psfDisp:0, $
            mtfDisp:0, $
            phaseDisp:0, $
            sliders:sliders, $
            open_file:open_file, $
            file_conv:file_conv, $
            readme:readme, $
            zreset:zreset, $
            spupil:spupil, $
            zpath:'./', $
            impath:'./', $
            strehl_ratio:strehl_ratio, $
            diffmax:diffmax, $
            rms_phase_rad:rms_phase_rad, $  
            lambda0:lambda0, $   
            wavelength_nm:wavelength_nm, $       
            apdraw:apdraw}

    ;; make a pointer and put it into the widget base
    infoptr = ptr_new(info)
    widget_control, base, set_uvalue = infoptr
  endelse

end
